--
--
--    Copyright (C) 2015  Henry Weidman
--
--    henry@weidmanenterprises.com
--
--    This is free software. You may share it or change
--    it. This software is distributed with no warranty. 
--
--
--
-- NAME
--		AdminTables
--
-- SYNOPSIS
--		psql -f Admin.sql
-- 
-- DESCRIPTION
--		SQL code to create the database tables and indexes
--
-- FILES
--		Admin.sql - This file
--
-- 

DROP TABLE admininfo ;

CREATE TABLE admininfo (
   command varchar,
   n SERIAL PRIMARY KEY
);

INSERT INTO admininfo VALUES ('BACKUP');
INSERT INTO admininfo VALUES ('RESTORE');
INSERT INTO admininfo VALUES ('REPORT');

DROP TABLE admincmd ;

CREATE TABLE admincmd (
   command varchar,
   cmdDate date,
   info    varchar,
   n SERIAL PRIMARY KEY
);

DROP TABLE countText ;

CREATE TABLE countText (
   textIn text
);

DROP TABLE report;

CREATE TABLE report (
   text	varchar
);


--
--
--
-- NAME
--	    AdminProcedures
--
-- SYNOPSIS
--	    psql -f Admin.sql
--
-- DESCRIPTION
--
--          The backup(), restore() and AdminCmd() procedures are defined here.
--          
-- FILES
--          Admin.sql - This file
--
-- 


CREATE OR REPLACE FUNCTION verifyDump() RETURNS int AS $$
DECLARE
  recCount int := 0;
  strA     date;
  strB     varchar;
  status   int := 1;
BEGIN
     strB := 'classdbBackup performed';
     strA := current_date;
     INSERT INTO admincmd VALUES ('pg_dump', strA, strB);

     return status;
END;
$$ LANGUAGE plpgsql;



--
-- If any .csv file cannot be read, postgres throws an exception
-- which is handled here.  If a file can be read, but is empty,
-- the code detects that the file is empty. 
--
CREATE OR REPLACE FUNCTION fileNotEmpty() RETURNS int AS $$
DECLARE
  status	int := 0;
  recCount	int := 0;
BEGIN

  --
  -- Start of exception handler
  --
  BEGIN
     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/members.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/address.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/phone.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/email.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

  EXCEPTION
     WHEN OTHERS THEN status := -2;
  END;
  --
  -- End of exception handler
  --

  return status;

END;
$$ LANGUAGE plpgsql;



--
-- If any .csv file cannot be read, postgres throws an exception
-- which is handled here.  If a file can be read, but is empty,
-- the code detects that the file is empty. 
--
CREATE OR REPLACE FUNCTION flashfilesNotEmpty() RETURNS int AS $$
DECLARE
  status	int := 0;
  recCount	int := 0;
BEGIN

  --
  -- Start of exception handler
  --
  BEGIN
     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/decks.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

     DELETE FROM countText;
     COPY countText FROM '/home/classdb/sql/indexcards.csv';
     SELECT count(*) INTO recCount FROM countText;
     IF recCount = 0
     THEN
       status := -1;
     END IF;

  EXCEPTION
     WHEN OTHERS THEN status := -2;
  END;
  --
  -- End of exception handler
  --

  return status;

END;
$$ LANGUAGE plpgsql;




CREATE OR REPLACE FUNCTION backup() RETURNS int AS $$
DECLARE
  status int := 0;

BEGIN
   COPY members TO '/home/classdb/sql/members.csv' WITH DELIMITER AS '~';
   COPY address TO '/home/classdb/sql/address.csv' WITH DELIMITER AS '~';
   COPY phone TO '/home/classdb/sql/phone.csv' WITH DELIMITER AS '~';
   COPY email TO '/home/classdb/sql/email.csv' WITH DELIMITER AS '~';
   COPY decks TO '/home/classdb/sql/decks.csv' WITH DELIMITER AS '~';
   COPY indexcards TO '/home/classdb/sql/indexcards.csv' WITH DELIMITER AS '~';
   return status;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION restore() RETURNS int AS $$
DECLARE
  status	int := 0;
  fileCount	int := 0;
  recCount	int := 0;
  startNum	int := 1000;

BEGIN

   status := fileNotEmpty();

   IF status = 0
   THEN
     DELETE FROM members;
     COPY members FROM '/home/classdb/sql/members.csv' WITH DELIMITER AS '~';

     SELECT max(num) INTO startNum FROM members;
     PERFORM setval('members_num_seq', startNum);


     DELETE FROM address;
     COPY address FROM '/home/classdb/sql/address.csv' WITH DELIMITER AS '~';
     DELETE FROM phone;
     COPY phone FROM '/home/classdb/sql/phone.csv' WITH DELIMITER AS '~';
     DELETE FROM email;
     COPY email FROM '/home/classdb/sql/email.csv' WITH DELIMITER AS '~';
   END IF;

   return status;
END;
$$ LANGUAGE plpgsql;


CREATE OR REPLACE FUNCTION restoreFlashCards() RETURNS int AS $$
DECLARE
  status	int := 0;
  fileCount	int := 0;
  recCount	int := 0;
  startNum	int := 1000;

BEGIN

   status := flashfilesNotEmpty();

   IF status = 0
   THEN
     DROP TRIGGER IF EXISTS deckDelete ON decks CASCADE;

     DELETE FROM indexcards;

     DELETE FROM decks;

     COPY decks FROM '/home/classdb/sql/decks.csv' WITH DELIMITER AS '~';

     COPY indexcards FROM '/home/classdb/sql/indexcards.csv' WITH DELIMITER AS '~';

     CREATE TRIGGER deckDelete BEFORE DELETE ON decks
        FOR EACH ROW EXECUTE PROCEDURE deckDel();

   END IF;


   return status;
END;
$$ LANGUAGE plpgsql;


--
-- Admin Command Procedure
--
CREATE OR REPLACE FUNCTION AdminCmd() RETURNS trigger AS $AdminCmd$
DECLARE
    ret int;

    BEGIN
        --raise notice 'AdminCmd()';
        
        ret := 1;

        CASE UPPER(NEW.command)
           WHEN 'BACKUP' THEN 
             ret := backup();
             NEW.INFO := 'BACKUP DONE';  
           WHEN 'RESTORE' THEN 
             ret := restore();
             ret := restoreFlashCards();
             NEW.INFO := 'RESTORE DONE'; 
           WHEN 'REPORT' THEN
             ret := membersReport(); 
             NEW.INFO := 'REPORT DONE';
           WHEN 'PG_DUMP' THEN
             ret := 1;
           ELSE
             NEW.info := 'Valid Commands are: BACKUP, RESTORE, REPORT';
        END CASE;

        IF ret < 0 
        THEN
          NEW.info := '***** One or more csv file is empty';
        END IF;
        
        
        --raise notice 'current_date';

        NEW.cmdDate := current_date;

       

        return NEW;
    END;
$AdminCmd$ LANGUAGE plpgsql;

CREATE TRIGGER AdminCmd BEFORE INSERT ON admincmd
    FOR EACH ROW EXECUTE PROCEDURE AdminCmd();

--
-- create the members report
--
CREATE OR REPLACE FUNCTION membersReport() RETURNS integer AS $$
DECLARE
  status 	int;
  txt    	varchar;
  memRec 	RECORD;
  addressRec	RECORD;
  phoneRec      RECORD;
  emailRec      RECORD;
BEGIN
  status := 1;

  DELETE FROM report;

  FOR memRec IN SELECT * FROM members LOOP
     txt := memRec.num || ' ' || memRec.firstname || ' ' || memRec.lastname;
     INSERT INTO report VALUES (txt);

     SELECT * FROM address INTO addressRec WHERE address.anum = memRec.num;
     txt := addressRec.street || ', ' || addressRec.city || ', ' || addressRec.state || ', ' || addressRec.zip;
     INSERT INTO report VALUES (txt);

     FOR phoneRec IN SELECT * FROM phone WHERE phone.pnum = memRec.num LOOP
        txt := phoneRec.phoneNumber;
        INSERT INTO report VALUES (txt);
     END LOOP;

     FOR emailRec IN SELECT * FROM email WHERE email.enum = memRec.num LOOP
        txt := emailRec.emailAddr;
        INSERT INTO report VALUES (txt);
     END LOOP;

     INSERT INTO report VALUES ('  ');

 
  END LOOP;

       COPY report TO '/home/classdb/sql/report.txt';


  return status;
END;
$$ LANGUAGE plpgsql;



