--
--
--    Copyright (C) 2015  Henry Weidman
--
--    henry@weidmanenterprises.com
--
--    This is free software. You may share it or change
--    it. This software is distributed with no warranty.
--   
--
--
--
--
-- NAME
--	MemberTables
--
-- SYNOPSIS
--	psql -f Members.sql
-- 
-- DESCRIPTION
--	SQL code to create the database tables and indexes 
--
--
--
-- FILES
--	File Members.sql - This file
--
-- 

DROP TABLE members CASCADE;

CREATE TABLE members (
   num SERIAL,
   FirstName varchar,
   LastName  varchar
);

ALTER TABLE members ADD PRIMARY KEY (num);

CREATE UNIQUE INDEX member_num_index ON members USING  btree (num);

ALTER SEQUENCE members_num_seq RESTART WITH 1000;

DROP TABLE address;

CREATE TABLE address (
   anum int REFERENCES members(num),
   street  varchar,
   city    varchar,
   state   varchar,
   zip     varchar
);


CREATE UNIQUE INDEX address_index ON address USING btree(anum);
ALTER TABLE address ADD PRIMARY KEY USING INDEX address_index;

DROP TABLE phone;

CREATE TABLE phone (
   pnum int REFERENCES members(num),
   snum SERIAL,
   PhoneNumber   varchar
);


CREATE UNIQUE INDEX phone_table_index ON phone USING btree(pnum, snum);
ALTER TABLE phone ADD PRIMARY KEY USING INDEX phone_table_index;

DROP TABLE email;

CREATE TABLE email (
   enum int REFERENCES members(num),
   snum SERIAL,
   emailAddr   varchar
);


CREATE UNIQUE INDEX email_table_index ON email USING btree(enum, snum);
ALTER TABLE email ADD PRIMARY KEY USING INDEX email_table_index;



--
--
-- NAME
--	    MemberProcedures
--
-- SYNOPSIS
--	    psql -f Members.sql
--
-- DESCRIPTION
--
--          The NewMember trigger inserts a new record into the address, phone, 
--          and email tables with the new members member number.  This is performed
--          automatically whenever a new member is added to the member table.
--
--          It also automatically copies the address and phone number from the 
--          first member found with the same last name.
--
--          The DeleteMember trigger deletes all of the records for the selected 
--          member.
--          
--          NewPhone formats the phone number 7035551212 as (703) 555-1212
--          Phone numbers that contain any alpha characters are not formatted as in:
--          "Cell (571) 555-1212" will be stored as is in the phone table
-- 
-- FILES
--          MemberProcedures.sql - This file
--
-- 


--
-- returns true only if all characters are numeric
--
CREATE OR REPLACE FUNCTION isNumeric(text) RETURNS boolean AS $$
DECLARE
  status boolean;
BEGIN
  SELECT $1 ~ '^[0-9]+$' INTO status;
  return status;
END;
$$ LANGUAGE plpgsql;

--
-- NewPhone formats the phone number 7035551212 as (703) 555-1212
-- Phone numbers that contain any alpha characters are not formatted as in:
--   "Cell (571) 555-1212" will be stored as is in the phone table
-- 
CREATE OR REPLACE FUNCTION NewPhone() RETURNS trigger AS $NewPhone$
DECLARE
  formatPhone varchar;

BEGIN

  -- raise notice 'New Phone: %', NEW.PhoneNumber;
  IF isNumeric(NEW.PhoneNumber)
  THEN
    formatPhone := '(' || substring(NEW.PhoneNumber from 1 for 3) || ') ' || substring(NEW.PhoneNumber from 4 for 3) ||
                   '-' || substring(NEW.PhoneNumber from 7 for 4);
    NEW.PhoneNumber := formatPhone;
  END IF;

  -- raise notice 'New Phone: %', formatPhone;
  -- raise notice 'New Phone: %', NEW.PhoneNumber;
  -- raise notice 'New Phone: Finished';

  RETURN NEW;
END;
$NewPhone$ LANGUAGE plpgsql;


CREATE TRIGGER NewPhone BEFORE INSERT ON phone
    FOR EACH ROW EXECUTE PROCEDURE NewPhone();



CREATE TRIGGER UpdPhone BEFORE UPDATE ON phone
    FOR EACH ROW EXECUTE PROCEDURE NewPhone();

--
-- New Member Procedure
--
CREATE OR REPLACE FUNCTION NewMember() RETURNS trigger AS $NewMember$
DECLARE

  n int;
  recCount int;
  lastNm varchar;
  firstNm varchar;
  addressRec RECORD;
  phoneRec RECORD;

    BEGIN

        -- raise notice 'BEGIN %', NEW.num;

        IF NEW.num IS NULL THEN
            RAISE EXCEPTION 'member number is null';
        END IF;
        
        -- raise notice 'num %', NEW.num;

        SELECT firstname INTO firstNm FROM members WHERE members.num=NEW.num;

        INSERT INTO address VALUES(NEW.num, 'street', 'city', 'NY', '12345');
        INSERT INTO phone VALUES(NEW.num, DEFAULT, '');
        INSERT INTO email VALUES(NEW.num, DEFAULT, firstNm);

        -- raise notice 'Inserts Done %', NEW.num;

        lastNm := UPPER(NEW.lastname);

        SELECT count(*) INTO n FROM members WHERE UPPER(lastname) = lastNm;
         -- raise notice 'num %', n;

        IF n > 0 
        THEN
          SELECT min(members.num) INTO n FROM members WHERE UPPER(lastname) = lastNm;
          SELECT * INTO addressRec FROM address WHERE anum = n;
          UPDATE address SET street = addressRec.street WHERE anum = NEW.num;
          UPDATE address SET city = addressRec.city WHERE anum = NEW.num;
          UPDATE address SET state = addressRec.state WHERE anum = NEW.num;
          UPDATE address SET zip = addressRec.zip WHERE anum = NEW.num;

          recCount := 0;
          FOR phoneRec IN SELECT * FROM phone WHERE pnum = n
          LOOP
            IF recCount = 0
            THEN
              UPDATE phone SET phoneNumber = phoneRec.phoneNumber WHERE pnum = NEW.num;
            END IF;
            recCount := recCount + 1;
          END LOOP;

        END IF;

        RETURN NEW;
    END;
$NewMember$ LANGUAGE plpgsql;


CREATE TRIGGER NewMember AFTER INSERT ON members
    FOR EACH ROW EXECUTE PROCEDURE NewMember();



--
-- Delete Member Procedure
--
CREATE OR REPLACE FUNCTION DeleteMember() RETURNS trigger AS $DeleteMember$
DECLARE

  n int;

    BEGIN
        n := OLD.num;

        -- raise notice 'Delete %', OLD.num;

        DELETE FROM address WHERE anum = n;
       
        DELETE FROM phone WHERE pnum = n;
     
        DELETE FROM email WHERE enum = n;

        RETURN OLD;
    END;

$DeleteMember$ LANGUAGE plpgsql;


CREATE TRIGGER DeleteMember BEFORE DELETE ON members
    FOR EACH ROW EXECUTE PROCEDURE DeleteMember();










